@@ -19,7 +19,7 @@
  *
  * @brief  ImageInfo structure and functions
  *
- * @author Michał Bąbik <michalb1981@o2.pl>
+ * @author Michal Babik <michal.babik@pm.me>
  */
 #include <stdio.h>
 #include <err.h>
@@ -61,9 +61,7 @@ imageinfo_new (void)
 {
     ImageInfo *ii_res;
 
-    ii_res = malloc (sizeof (ImageInfo));
-
-    if (ii_res == NULL)
+    if ((ii_res = malloc (sizeof (ImageInfo))) == NULL)
         err (EXIT_FAILURE, NULL);
 
     imageinfo_init (ii_res);
