@@ -19,17 +19,16 @@
  *
  * Automatic wallpaper changer
  *
- * @date June 03, 2020
+ * @date June 22, 2020
  *
- * @version 1.4.9
+ * @version 1.5.0
  *
- * @author Michał Bąbik <michalb1981@o2.pl>
+ * @author Michal Babik <michal.babik@pm.me>
  */
 #include <stdio.h>
 #include <stdint.h>
 #include <gtk/gtk.h>
 #include "dialogdata.h"
-#include "settlist.h"
 #include "setts.h"
 #include "wpset.h"
 #include "imgs.h"
@@ -43,8 +42,8 @@
 #include "defs.h"
 #include "hashfun.h"
 #include "dmfn.h"
-#include "astart.h"
 #include "wmsfn.h"
+#include "deffiles.h"
 /*----------------------------------------------------------------------------*/
 /**
  * @fn  static uint32_t get_wallpaper_ch_interval (const DialogData *dd_data)
@@ -55,67 +54,48 @@
  * @return        Change interval value
  *
  * @fn  static void set_wallpaper_ch_interval (const DialogData *dd_data,
- *                                             const uint32_t    ui_val)
+ *                                             const uint32_t    i_val)
  *
  * @brief  Set wallpaper change interval value to widgets
  *
  * @param[in,out] dd_data  DialogData object with settings and widget data
- * @param[in]     ui_val   Change interval value to set
+ * @param[in]     i_val    Change interval value to set
  * @return        none
  */
 /*----------------------------------------------------------------------------*/
 static uint32_t    get_wallpaper_ch_interval   (const DialogData  *dd_data);
 
 static void        set_wallpaper_ch_interval   (const DialogData  *dd_data,
-                                                const uint32_t     ui_val);
+                                                const uint32_t     i_val);
 /*----------------------------------------------------------------------------*/
 /**
- * @fn  static void get_wallpaper_list (GtkWidget *gw_view, SettList  *st_list)
- *
- * @brief  Insert list of wallpapers from treeview to SettList array.
- *
- * @param[in]  gw_view  TreeView to read file list
- * @param[out] st_list  List with settings to insert file list
- * @return     none
- *
- * @fn  static void set_wallpaper_list (const SettList *sl_walls,
- *                                      GtkWidget      *gw_view)
- *
- * @brief  Insert list of wallpapers from SettList array to treeview.
- *
- * @param[in]  sl_walls List with wallpaper file paths
- * @param[out] gw_view  TreeView to insert file list
- * @return     none
- */
-/*----------------------------------------------------------------------------*/
-static void        get_wallpaper_list          (GtkWidget         *gw_view,
-                                                SettList          *st_list);
-
-static void        set_wallpaper_list          (const SettList    *sl_walls,
-                                                GtkWidget         *gw_view);
-/*----------------------------------------------------------------------------*/
-/**
- * @fn  static SettList widgets_get_settings (const DialogData *dd_data)
+ * @fn  static Setting * widgets_get_settings (const DialogData  *dd_data)
  *
  * @brief  Read settings from widgets and store them in WallSett object.
  *
  * @param[in,out] dd_data  DialogData object with settings and widget data
- * @return        none
+ * @return        List of settings
  *
- * @fn  static void widgets_set_settings (const DialogData *dd_data,
- *                                        const SettList   *st_list)
+ * @fn  static void widgets_set_settings (const DialogData  *dd_data,
+ *                                        Setting           *st_settings,
+ *                                        Setting           *st_wmlist,
+ *                                        int               *i_err)
  *
- * @brief  Loading data from SettList list of settings to program window.
+ * @brief  Loading data from list of settings to program window.
  *
- * @param[in,out] dd_data  DialogData object with settings and widget data
- * @param[in]     st_list  List with settings
+ * @param[in,out] dd_data      DialogData object with settings and widget data
+ * @param[in]     st_settings  List with settings
+ * @param[in]     st_wmlist    List with window manager info
+ * @param[out]    i_err        Error output
  * @return        none
  */
 /*----------------------------------------------------------------------------*/
-static SettList  * widgets_get_settings        (const DialogData  *dd_data);
+static Setting   * widgets_get_settings        (const DialogData  *dd_data);
 
 static void        widgets_set_settings        (const DialogData  *dd_data,
-                                                const SettList    *st_list);
+                                                Setting           *st_settings,
+                                                Setting           *st_wmlist,
+                                                int               *i_err);
 /*----------------------------------------------------------------------------*/
 /**
  * @brief  Sets statusbar info about actual config file.
@@ -145,28 +125,28 @@ static gboolean    event_on_delete             (GtkWidget         *window,
 /**
  * @fn  static void event_add_img_pressed (const DialogData *dd_data)
  *
- * @brief  Add images button pressed.
+ * @brief  "Add images" button pressed.
  *
  * @param[in,out] dd_data  DialogData object with settings and widget data
  * @return        none
  *
  * @fn  static void event_add_img_dir_pressed (const DialogData *dd_data)
  *
- * @brief  Add images from folder button pressed.
+ * @brief  "Add images from folder" button pressed.
  *
  * @param[in,out] dd_data  DialogData object with settings and widget data
  * @return        none
  *
  * @fn  static void event_set_wallpaper_pressed (const DialogData *dd_data)
  *
- * @brief  Set wallpaper button pressed.
+ * @brief  "Set wallpaper" button pressed.
  *
  * @param[in,out] dd_data  DialogData object with settings and widget data
  * @return        none
  *
  * @fn  static void event_save_settings_pressed (const DialogData *dd_data)
  *
- * @brief  Save settings button pressed.
+ * @brief  "Save settings" button pressed.
  *
  * @param[in,out] dd_data  DialogData object with settings and widget data
  * @return none
@@ -247,7 +227,7 @@ static gboolean    event_treeview_key_press    (GtkWidget         *widget,
 static void        event_interval_changed       (GtkSpinButton     *spin_button,
                                                  const DialogData  *dd_data);
 
-static void        event_command_button_pressed (const DialogData *dd_data);
+static void        event_command_button_pressed (const DialogData  *dd_data);
 
 static void        event_start_daemon_pressed   (DialogData        *dd_data);
 
@@ -355,10 +335,10 @@ get_wallpaper_ch_interval (const DialogData *dd_data)
  * @brief  Set wallpaper change interval value to widgets
  */
 static void
-set_wallpaper_ch_interval (const DialogData    *dd_data,
-                           const uint32_t ui_val)
+set_wallpaper_ch_interval (const DialogData *dd_data,
+                           const uint32_t    i_val)
 {
-    uint32_t ui_tmp = ui_val; /* Temp inteval value */
+    uint32_t ui_tmp = i_val; /* Temp inteval value */
 
     /* Check if minutes value can be displayed as hours */
     if ((ui_tmp / 60 >= 1) && (ui_tmp % 60 == 0)) {
@@ -376,99 +356,53 @@ set_wallpaper_ch_interval (const DialogData    *dd_data,
                                (double) ui_tmp);
 }
 /*----------------------------------------------------------------------------*/
-/**
- * @brief  Insert list of wallpapers from treeview to SettList array.
- */
-static void
-get_wallpaper_list (GtkWidget *gw_view,
-                    SettList  *st_list)
-{
-    ImageInfo  *ii_info;           /* ImageInfo item for data from tree view */
-    Setting    *st_sett;           /* Setting with file name for st_list */
-    GSList     *gsl_iinfo1 = NULL; /* List of ImageInfo data */
-    GSList     *gsl_iinfo  = NULL; /* Temp list for data */
-    const char *s_val      = NULL; /* Full file path */
-
-    /* get ImageInfo list of TreeView files */
-    gsl_iinfo1 = treeview_get_data (gw_view);
-    gsl_iinfo  = gsl_iinfo1;
-    st_sett    = setting_new_array (get_setting_name (SETTING_WALL_ARRAY));
-
-    stlist_insert_setting (st_list, st_sett);
-
-    while (gsl_iinfo != NULL) {
-
-        ii_info = gsl_iinfo->data;
-        s_val   = ii_info->s_full_path;
-        st_sett = setting_new_string (s_val, NULL);
-        
-        stlist_insert_setting_to_array (st_list, st_sett,
-                                        get_setting_name (SETTING_WALL_ARRAY));
-        gsl_iinfo = gsl_iinfo->next;
-    }
-    g_slist_free_full (gsl_iinfo1, (GDestroyNotify) imageinfo_free);
-}
-/*----------------------------------------------------------------------------*/
-/**
- * @brief  Insert list of wallpapers from SettList array to treeview.
- */
-static void
-set_wallpaper_list (const SettList *sl_walls,
-                    GtkWidget      *gw_view)
-{
-    treeview_add_items_settlist (gw_view, sl_walls);
-}
-/*----------------------------------------------------------------------------*/
 /**
  * @brief  Read settings from widgets and store them in WallSett object.
  */
-static SettList *
+static Setting *
 widgets_get_settings (const DialogData *dd_data)
 {
-    SettList   *st_list;       /* List of settings */
+    Setting    *st_settings;
     Setting    *st_sett;       /* Setting to add */
     const char *s_val  = NULL; /* Value for a string setting */
-    uint32_t    ui_val = 0;    /* Value for an integer setting */
+    int64_t     i_val  = 0;    /* Value for an integer setting */
 
-    st_list = stlist_new_list ();
+    st_settings = setting_new_setting ("Settings");
 
     /* Get wallpaper list from treeview and add to settlist */
-    get_wallpaper_list (dd_data->gw_view, st_list);
+    st_sett = setting_new_array (get_setting_name (SETTING_WALL_ARRAY));
+    treeview_get_setting_data (dd_data->gw_view, st_sett);
+    setting_add_child (st_settings, st_sett);
 
     /* Get random wallpaper setting */
-    ui_val = (uint32_t) gtk_toggle_button_get_active (
+    i_val = (int64_t) gtk_toggle_button_get_active (
                 GTK_TOGGLE_BUTTON (dd_data->gw_random));
-    st_sett = setting_new_uint32 (
-            ui_val, get_setting_name (SETTING_RANDOM_OPT));
-    stlist_insert_setting (st_list, st_sett);
+    st_sett = setting_new_int (get_setting_name (SETTING_RANDOM_OPT), i_val);
+    setting_add_child (st_settings, st_sett);
 
     /* Get last used wallpaper on start setting */
-    ui_val = (uint32_t) gtk_toggle_button_get_active (
+    i_val = (int64_t) gtk_toggle_button_get_active (
                 GTK_TOGGLE_BUTTON (dd_data->gw_lastused));
-    st_sett = setting_new_uint32 (
-            ui_val, get_setting_name (SETTING_LAST_USED_OPT));
-    stlist_insert_setting (st_list, st_sett);
+    st_sett = setting_new_int (get_setting_name (SETTING_LAST_USED_OPT), i_val);
+    setting_add_child (st_settings, st_sett);
 
     /* Get time align setting */
-    ui_val = (uint32_t) gtk_toggle_button_get_active (
+    i_val = (int64_t) gtk_toggle_button_get_active (
                 GTK_TOGGLE_BUTTON (dd_data->gw_timealign));
-    st_sett = setting_new_uint32 (
-            ui_val, get_setting_name (SETTING_TIME_ALIGN_OPT));
-    stlist_insert_setting (st_list, st_sett);
+    st_sett = setting_new_int (get_setting_name (SETTING_TIME_ALIGN_OPT), i_val);
+    setting_add_child (st_settings, st_sett);
 
     /* Get wallpaper change interval setting */
-    ui_val = get_wallpaper_ch_interval (dd_data);
-    st_sett = setting_new_uint32 (
-            ui_val, get_setting_name (SETTING_INTERVAL_VAL));
-    stlist_insert_setting (st_list, st_sett);
+    i_val = (int64_t) get_wallpaper_ch_interval (dd_data);
+    st_sett = setting_new_int (get_setting_name (SETTING_INTERVAL_VAL), i_val);
+    setting_add_child (st_settings, st_sett);
 
     /* Get wallpaper set command */
     s_val = gtk_entry_get_text (GTK_ENTRY (dd_data->gw_command));
-    st_sett = setting_new_string (
-            s_val, get_setting_name (SETTING_BG_CMD));
-    stlist_insert_setting (st_list, st_sett);
+    st_sett = setting_new_string (get_setting_name (SETTING_BG_CMD), s_val);
+    setting_add_child (st_settings, st_sett);
 
-    return st_list;
+    return st_settings;
 }
 /*----------------------------------------------------------------------------*/
 /**
@@ -476,85 +410,88 @@ widgets_get_settings (const DialogData *dd_data)
  */
 static void
 widgets_set_settings (const DialogData *dd_data,
-                      const SettList   *st_list)
+                      Setting          *st_settings,
+                      Setting          *st_wmlist,
+                      int              *i_err)
 {
-    Setting    *st_sett;            /* Setting to read */
-    SettList   *sl_walls;           /* List of wallpapers */
-    int         i_w         = 0;    /* Window width value */
-    int         i_h         = 0;    /* Window height value */
-    const char *s_setts_cmd = NULL; /* Command from settings */
-    const char *s_cmd       = NULL; /* Command from detection */
+    Setting    *st_item    = NULL; /* Setting to read */
+    int         i_w        = 0;    /* Window width value */
+    int         i_h        = 0;    /* Window height value */
+    const char *s_lastused = NULL; /* Last used wallpapet string */
+    char       *s_cmd      = NULL; /* Wallpaper set command */
 
-    /* Set background set command */
-    st_sett = stlist_get_setting_with_name (
-            st_list, get_setting_name (SETTING_BG_CMD));
-    if (st_sett != NULL) {
-        s_setts_cmd = setting_get_string (st_sett);
-        if ((s_setts_cmd != NULL && s_setts_cmd[0] == '\0') ||
-                s_setts_cmd == NULL) {
+    *i_err = ERR_OK;
 
-            s_cmd = cmddialog_find_command ();
-            if (s_cmd == NULL) {
-                gtk_entry_set_text (GTK_ENTRY (dd_data->gw_command),
-                        DEFAULT_BG_CMD);
-            }
-            else {
-                gtk_entry_set_text (GTK_ENTRY (dd_data->gw_command), s_cmd);
-            }
-        }
-        else {
-            gtk_entry_set_text (GTK_ENTRY (dd_data->gw_command), s_setts_cmd);
-        }
+    /* Check window managers and set wallpaper set command */
+    s_cmd = wms_get_wallpaper_command (dd_data->s_cfg_file,
+                                       st_settings,
+                                       st_wmlist,
+                                       i_err);
+    if (*i_err != ERR_OK) {
+        free (s_cmd);
+        return;
     }
+    gtk_entry_set_text (GTK_ENTRY (dd_data->gw_command), s_cmd);
+    free (s_cmd);
+
     /* Set last used wallpaper on start setting */
-    st_sett = stlist_get_setting_with_name (
-            st_list, get_setting_name (SETTING_LAST_USED_OPT));
-    if (st_sett != NULL) {
+    st_item = settings_find (st_settings,
+                             get_setting_name (SETTING_LAST_USED_OPT));
+    if (st_item != NULL) {
         gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dd_data->gw_lastused),
-                                      (gboolean) setting_get_uint32 (st_sett));
+                                      (gboolean) setting_get_int (st_item));
     }
     /* Set minutes/hours of wallpaper change interval */
-    st_sett = stlist_get_setting_with_name (
-            st_list, get_setting_name (SETTING_INTERVAL_VAL));
-    if (st_sett != NULL) {
-        set_wallpaper_ch_interval (dd_data, setting_get_uint32 (st_sett));
+    st_item = settings_find (st_settings,
+                             get_setting_name (SETTING_INTERVAL_VAL));
+    if (st_item != NULL) {
+        set_wallpaper_ch_interval (dd_data,
+                                   (uint32_t) setting_get_int (st_item));
     }
     /* Set wallpaper random choose setting */
-    st_sett = stlist_get_setting_with_name (
-            st_list, get_setting_name (SETTING_RANDOM_OPT));
-    if (st_sett != NULL) {
+    st_item = settings_find (st_settings,
+                             get_setting_name (SETTING_RANDOM_OPT));
+    if (st_item != NULL) {
         gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dd_data->gw_random),
-                                      (gboolean) setting_get_uint32 (st_sett));
+                                      (gboolean) setting_get_int (st_item));
     }
     /* Set time align setting */
-    st_sett = stlist_get_setting_with_name (
-            st_list, get_setting_name (SETTING_TIME_ALIGN_OPT));
-    if (st_sett != NULL) {
+    st_item = settings_find (st_settings,
+                             get_setting_name (SETTING_TIME_ALIGN_OPT));
+    if (st_item != NULL) {
         gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dd_data->gw_timealign),
-                                      (gboolean) setting_get_uint32 (st_sett));
+                                      (gboolean) setting_get_int (st_item));
     }
     /* Get wallpaper list and add to treeview */
-    sl_walls = stlist_get_settings_in_array_name_p (st_list,
-            get_setting_name (SETTING_WALL_ARRAY));
-
-    if (sl_walls != NULL) {
-        set_wallpaper_list (sl_walls, dd_data->gw_view);
-        stlist_free_p (sl_walls);
+    st_item = settings_find (st_settings,
+                             get_setting_name (SETTING_WALL_ARRAY));
+    if (st_item != NULL) {
+        treeview_add_items_setting (dd_data->gw_view,
+                                    setting_get_child (st_item));
     }
-
     /* Set window dimensions */
-    st_sett = stlist_get_setting_with_name (
-            st_list, get_setting_name (SETTING_WIN_WIDTH));
-    if (st_sett != NULL) {
-        i_w = (int) setting_get_uint32 (st_sett);
+    st_item = settings_find (st_settings, get_setting_name (SETTING_WIN_WIDTH));
+    if (st_item != NULL) {
+        i_w = (int) setting_get_int (st_item);
     }
-    st_sett = stlist_get_setting_with_name (
-            st_list, get_setting_name (SETTING_WIN_HEIGHT));
-    if (st_sett != NULL) {
-        i_h = (int) setting_get_uint32 (st_sett);
+    st_item = settings_find (st_settings,
+                             get_setting_name (SETTING_WIN_HEIGHT));
+    if (st_item != NULL) {
+        i_h = (int) setting_get_int (st_item);
     }
- 
     gtk_window_set_default_size (GTK_WINDOW (dd_data->gw_window), i_w, i_h);
+
+    /* Check if there is last used wallpaper info, if it is select it on
+     * list and make a preview image */
+    st_item = settings_find (st_settings,
+                             get_setting_name (SETTING_LAST_USED_STR));
+    if (st_item != NULL)
+        s_lastused = setting_get_string (st_item);
+
+    if (s_lastused != NULL) {
+        preview_from_file (dd_data->gw_imgprev, s_lastused);
+        treeview_find_select_item (dd_data->gw_view, s_lastused);
+    }
 }
 /*----------------------------------------------------------------------------*/
 /**
@@ -575,7 +512,7 @@ statusbar_push_config_info (const DialogData *dd_data)
 }
 /*----------------------------------------------------------------------------*/
 /**
- * @brief  Add images button pressed.
+ * @brief  "Add images" button pressed.
  */
 static void
 event_add_img_pressed (const DialogData *dd_data)
@@ -594,7 +531,7 @@ event_add_img_pressed (const DialogData *dd_data)
 }
 /*----------------------------------------------------------------------------*/
 /**
- * @brief  Add images from folder button pressed.
+ * @brief  "Add images from folder" button pressed.
  */
 static void
 event_add_img_dir_pressed (const DialogData *dd_data)
@@ -619,7 +556,7 @@ event_add_img_dir_pressed (const DialogData *dd_data)
 }
 /*----------------------------------------------------------------------------*/
 /**
- * @brief  Set wallpaper button pressed.
+ * @brief  "Set wallpaper" button pressed.
  */
 static void
 event_set_wallpaper_pressed (const DialogData *dd_data)
@@ -655,18 +592,18 @@ event_set_wallpaper_pressed (const DialogData *dd_data)
 }
 /*----------------------------------------------------------------------------*/
 /**
- * @brief  Save settings button pressed.
+ * @brief  "Save settings" button pressed.
  */
 static void
 event_save_settings_pressed (const DialogData *dd_data)
 {
-    SettList *st_list;   /* List of settings */
-    int       i_err = 0; /* Error value */
+    Setting *st_settings; /* List of settings */
+    int       i_err = 0;  /* Error value */
 
-    st_list = widgets_get_settings (dd_data);
-    i_err   = settings_check_update_file (st_list,
-                                          dialogdata_get_cfg_file (dd_data));
-    stlist_free (st_list);
+    st_settings = widgets_get_settings (dd_data);
+    i_err = setts_check_update_file (dialogdata_get_cfg_file (dd_data),
+                                     st_settings);
+    settings_free_all (st_settings);
 
     if (i_err != ERR_OK) {
         message_dialog_error (dd_data->gw_window, err_get_message (i_err));
@@ -790,15 +727,22 @@ event_command_button_pressed (const DialogData *dd_data)
     g_slist_free_full (gsl_files, (GDestroyNotify) imageinfo_free);
 }
 /*----------------------------------------------------------------------------*/
+/**
+ * @brief  Changed active state of autostart checkbox
+ *
+ * @param[in] togglebutton  Togglebutton which changed state
+ * @param[in] user_data     Data passed to function
+ * @return    none
+ */
 static void
 event_autostart_toggled (GtkToggleButton *togglebutton,
                          gpointer         user_data __attribute__ ((unused)))
 {
     if (gtk_toggle_button_get_active (togglebutton)) {
-        autostart_create ();
+        deffiles_autostart_create ();
     }
     else {
-        autostart_remove ();
+        deffiles_autostart_remove ();
     }
 }
 /*----------------------------------------------------------------------------*/
@@ -811,7 +755,7 @@ event_on_delete (GtkWidget        *window,
                  const DialogData *dd_data)
 {
     GtkWidget *dialog;        /* Question dialog */
-    SettList  *st_list;       /* List of settings */
+    Setting   *st_settings;   /* Program settings */
     char      *s_buff = NULL; /* Buffer for settings data */
     int        i_res  = 0;    /* Dialog result */
     int        i_err  = 0;    /* Error value */
@@ -820,11 +764,12 @@ event_on_delete (GtkWidget        *window,
 
     gtk_window_get_size (GTK_WINDOW (window), &i_w, &i_h);
 
-    st_list = widgets_get_settings (dd_data);
-    s_buff  = settings_check_update (st_list,
-                                     dialogdata_get_cfg_file (dd_data),
-                                     &i_err);
-    stlist_free (st_list);
+    st_settings = widgets_get_settings (dd_data);
+
+    s_buff  = setts_check_update (dialogdata_get_cfg_file (dd_data),
+                                  st_settings,
+                                  &i_err);
+    settings_free_all (st_settings);
 
     if (i_err != ERR_OK) {
         message_dialog_error (GTK_WINDOW (window), err_get_message (i_err));
@@ -841,8 +786,8 @@ event_on_delete (GtkWidget        *window,
         gtk_widget_destroy (dialog);
 
         if (i_res == GTK_RESPONSE_YES) {
-            i_err = settings_update_file (s_buff,
-                                          dialogdata_get_cfg_file (dd_data));
+            i_err = setts_update_file (dialogdata_get_cfg_file (dd_data),
+                                       s_buff);
             if (i_err != ERR_OK) {
                 message_dialog_error (GTK_WINDOW (window),
                                       err_get_message (i_err));
@@ -850,7 +795,7 @@ event_on_delete (GtkWidget        *window,
         }
         free (s_buff);
     }
-    settings_update_window_size (i_w, i_h, dialogdata_get_cfg_file(dd_data));
+    setts_update_window_size (dialogdata_get_cfg_file(dd_data), i_w, i_h);
     return FALSE;
 }
 /*----------------------------------------------------------------------------*/
@@ -932,10 +877,10 @@ create_image_button (const char   *s_label,
 
     gw_btn = gtk_button_new ();
 
-    if (s_label != NULL && strcmp (s_label, "") != 0)
+    if (s_label != NULL && s_label[0] != '\0')
         gtk_button_set_label (GTK_BUTTON (gw_btn), s_label);
 
-    if (s_hint != NULL && strcmp (s_hint, "") != 0)
+    if (s_hint != NULL && s_hint[0] != '\0')
         gtk_widget_set_tooltip_text (gw_btn, s_hint);
 
     if (i_but < W_IMG_COUNT && (gd_pix = get_image (i_but)) != NULL) {
@@ -1029,7 +974,8 @@ create_buttons_widget (GtkWidget  **gw_widget,
     g_signal_connect_swapped (gw_button,
                               "clicked",
                               G_CALLBACK (about_app_dialog), NULL);
-    gw_button = create_image_button (NULL, "Exit app", W_ICON_EXIT);
+    gw_button = create_image_button (NULL, "Exit configuration dialog",
+                                     W_ICON_EXIT);
     g_signal_connect_swapped (gw_button,
                               "clicked",
                               G_CALLBACK (gtk_window_close),
@@ -1086,8 +1032,7 @@ create_settings_widget (GtkWidget **gw_widget,
         " 10:15, first interval will be changed to 5 minutes so changes"
         " will appear on 10:20, 10:40, 11:00 ..."
         );
-    gtk_button_set_label (GTK_BUTTON (gw_align_button),
-                          "Align");
+    gtk_button_set_label (GTK_BUTTON (gw_align_button), "Align");
 
     /* Wallpaper set command entry */
     gw_command_label = gtk_label_new ("Background set command : ");
@@ -1098,7 +1043,7 @@ create_settings_widget (GtkWidget **gw_widget,
     gtk_widget_set_tooltip_markup (gw_command_entry,
         "This command will be executed to set background image\n"
         "e.g. <b>feh --bg-fill</b>\nFor more complex commands use <b>[F]</b>" 
-        " as a file name\ne.g. <b>feh --bg-fill [F]</b>");
+        " as a file name\ne.g. <b>feh --bg-fill \"[F]\"</b>");
     gw_command_button = gtk_button_new_with_label ("Select");
     /*
     gtk_widget_set_tooltip_text (gw_command_button,
@@ -1132,15 +1077,15 @@ create_settings_widget (GtkWidget **gw_widget,
     gw_autostart_button = gtk_check_button_new ();
     gtk_widget_set_tooltip_markup (gw_autostart_button, 
         "When <b>enabled</b> wchangerd daemon entry will be placed in "
-        "user's autostart directory."
+        "users autostart directory."
         "\nWhen <b>disabled</b> daemon entry will be removed from "
-        "user's autostart directory."
+        "users autostart directory."
         "\nThis will allow to start (restart actually) wchangerd daemon "
         " in XDG Autostart compatible window managers.");
     gtk_button_set_label (GTK_BUTTON (gw_autostart_button),
                           "Create autostart entry");
     gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (gw_autostart_button),
-                                  autostart_exists ());
+                                  deffiles_autostart_exists ());
     g_signal_connect (gw_autostart_button, "toggled",
                       G_CALLBACK (event_autostart_toggled), NULL);
 
@@ -1210,20 +1155,24 @@ create_daemon_widget (DialogData *dd_data)
     GtkWidget *gw_widget;
     GtkWidget *gw_label;
 
+    /* Grid to pack widgets */
     gw_widget = gtk_grid_new ();
     gtk_grid_set_row_spacing (GTK_GRID (gw_widget), 4);
     gtk_grid_set_column_spacing (GTK_GRID (gw_widget), 4);
 
-    gw_label  = gtk_label_new ("wchangerd");
+    /* Daemon name label */
+    gw_label = gtk_label_new ("wchangerd");
     gtk_label_set_xalign (GTK_LABEL (gw_label), 0.5);
 
     gtk_grid_attach (GTK_GRID (gw_widget), gw_label, 0, 0, 2, 1);
 
+    /* Daemon run status label */
     dd_data->gw_dm_label = gtk_label_new (" ");
     gtk_label_set_xalign (GTK_LABEL (dd_data->gw_dm_label), 0.5);
 
     gtk_grid_attach (GTK_GRID (gw_widget), dd_data->gw_dm_label, 0, 1, 2, 1);
 
+    /* Start daemon button */
     gw_button = create_image_button (NULL, "Start wchangerd daemon",
                                      W_ICON_PLAY);
     g_signal_connect_swapped (gw_button, "clicked",
@@ -1231,6 +1180,7 @@ create_daemon_widget (DialogData *dd_data)
 
     gtk_grid_attach (GTK_GRID (gw_widget), gw_button, 0, 2, 1, 1);
 
+    /* Stop daemon button */
     gw_button = create_image_button (NULL, "Stop wchangerd daemon",
                                      W_ICON_STOP);
     g_signal_connect_swapped (gw_button, "clicked",
@@ -1260,16 +1210,17 @@ activate (GtkApplication *app,
     GtkWidget  *gw_settings_widget; /* Setings for wallpaper changing */
     GtkWidget  *gw_box_main;        /* Main box to pack everything */
     GtkWidget  *gw_statusbar;       /* Bottom status bar */
-    SettList   *st_list;            /* List of settings */
-    Setting    *st_sett;            /* Setting object */
+    Setting    *st_settings;        /* Program settings */
+    Setting    *st_wm      = NULL;  /* Window manager info */
     GdkPixbuf  *gd_pix     = NULL;  /* Default widget icon */
-    const char *s_lastused = NULL;  /* Last used wallpaper path */
     int         i_err      = 0;     /* For error output */
 
+    /* Find config file and set config file name */
     dialogdata_do_config_file_stuff (dd_data);
 
     /* Image preview widget */
     gw_img_prev = gtk_image_new ();
+    dd_data->gw_imgprev = gw_img_prev;
 
     /* Main window */
     gw_window = gtk_application_window_new (app);
@@ -1284,7 +1235,7 @@ activate (GtkApplication *app,
         gtk_window_set_default_icon (gd_pix);
         g_object_unref (gd_pix);
     }
-
+    /* Treeview for background file list */
     create_tview (&gw_tview);
     dd_data->gw_view = gw_tview;
     g_signal_connect (gw_tview, "row-activated",
@@ -1360,35 +1311,44 @@ activate (GtkApplication *app,
     gtk_container_set_border_width (GTK_CONTAINER (gw_box_main), 10);
     gtk_container_add (GTK_CONTAINER (gw_window), gw_box_main);
 
-    st_list = settings_read (dialogdata_get_cfg_file (dd_data), &i_err);
+    /* Read program settings */
+    st_settings = setts_read (dialogdata_get_cfg_file (dd_data), &i_err);
+
+    /* Check window managers config file */
+    if (i_err == ERR_OK)
+        i_err = deffiles_wm_check_create ();
+
+    /* Get window manager data */
+    if (i_err == ERR_OK)
+        st_wm = wms_get_wm_info (&i_err);
+
     if (i_err != ERR_OK) {
         message_dialog_error (NULL, err_get_message (i_err));
-        stlist_free (st_list);
+        settings_free_all (st_settings);
+        settings_free_all (st_wm);
         g_application_quit (G_APPLICATION (app));
         return;
     }
 
-    settlist_check_defaults (st_list);
+    setts_check_defaults (st_settings);
 
-    widgets_set_settings (dd_data, st_list);
+    widgets_set_settings (dd_data,
+                          setting_get_child (st_settings),
+                          setting_get_child (st_wm),
+                          &i_err);
 
-    st_sett = stlist_get_setting_with_name (
-            st_list, get_setting_name (SETTING_LAST_USED_STR));
+    settings_free_all (st_wm);
+    settings_free_all (st_settings);
 
-    if (st_sett != NULL)
-        s_lastused = setting_get_string (st_sett);
-
-    /* Check if there is last used wallpaper info, if it is select it on
-     * list and make a preview image */
-    if (s_lastused != NULL) {
-        preview_from_file (gw_img_prev, s_lastused);
-        treeview_find_select_item (gw_tview, s_lastused);
+    if (i_err != ERR_OK) {
+        message_dialog_error (NULL, err_get_message (i_err));
+        g_application_quit (G_APPLICATION (app));
+        return;
     }
 
-    stlist_free (st_list);
-
+    /* Set info about wchangerd deamon presence */
     daemon_monitor (dd_data);
-
+    /* Add gtk thread for checking wchangerd deaemon process presence */
     gdk_threads_add_timeout_full (G_PRIORITY_LOW,
                                   3000,
                                   daemon_monitor,
